/************************************************/
/* Zpotov projekt do P009                    */
/* Marie Cackov <xborakov@fi.muni.cz>  		*/
/************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include "glut.h"

#define PI 3.141592654

void CloseTheWindow();

GLuint windowID;
GLuint menuID;
GLuint width  = 512;
GLuint height = 512;
GLboolean animace = GL_FALSE;
GLboolean kamera  = GL_FALSE;
GLboolean bocni  = GL_FALSE;

/* Definice materialu... */
GLfloat hodn1[] = { 0.1, 0.1, 0.1 };
GLfloat hodn2[] = { 0.1, 0.9, 0.1 };
GLfloat hodn3[] = { 0.4, 0.1, 0.1 };
GLfloat no_mat[] = { 0.2, 0.0, 0.0, 1.0 };
GLfloat mat_ambient[] = { 0.7, 0.7, 0.7, 1.0 };
GLfloat mat_ambient_color[] = { 0.8, 0.8, 0.2, 1.0 };
GLfloat mat_diffuse[] = { 0.1, 0.5, 0.8, 1.0 };
GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
GLfloat no_shininess[] = { 0.0 };
GLfloat low_shininess[] = { 5.0 };
GLfloat high_shininess[] = { 120.0 };
GLfloat mat_emission[] = {0.5, 0.2, 0.2, 0.0};

float krok = 0.08;
float dx = -0.6;
float dy = -0.6;
float dz = 0.0;
float otoceni = 90;
int pocet_otoceni = 0;
int pohyb = 0;

/* Promenne tykajici se pohybu po spirale */
float spx = 0.0;
float spy = 0.0;
float spz = 0.0;
float z_min = 0.1;
float z_max = 1.1;
float pocet_otacek = 4.0;
float polomer_spiraly = 0.3;
float i = 0, j = 0;

/* Promenne pro pohled kamery */
float ki = 0;
float kx = 0, ky = 0, kz = 0;
float polomer_kamery = 1.7;
 
void vytvor() {
  if (animace)
  /* Zpracuje pohyb jehlanu */
  switch (pohyb)
  {
  case 0:
    dx=dx+krok;
    if (dx>=0.6) { pocet_otoceni=0; pohyb++; }
    break;
  case 1:
    otoceni+=10;
    pocet_otoceni++;
    if (pocet_otoceni>11) pohyb++;
    break;
  case 2:
    dx=dx-(krok/2);
    dy=dy+krok;
    dz=dz+krok/4;
    if (dx<=0) { pocet_otoceni=0; pohyb++; }
    break;
  case 3:
    otoceni+=10;
    pocet_otoceni++;
    if (pocet_otoceni>11) pohyb++;
    break;
  case 4:
    dx=dx-(krok/2);
    dy=dy-krok;
    dz=dz-krok/4;
    if (dy<=-0.6) { pocet_otoceni=0; pohyb++; }
    break;     
  case 5:
    otoceni+=10;
    pocet_otoceni++;
    if (pocet_otoceni>11) pohyb=0;
    break;
  }

  /* Jehlan */
  glPushMatrix();
    glMaterialfv(GL_FRONT, GL_DIFFUSE, hodn1);
    glMaterialfv(GL_FRONT, GL_SPECULAR, hodn2);
    glMaterialfv(GL_FRONT, GL_SHININESS, high_shininess);
    glColor3d(0.50,0.50,0);

    glTranslated(dx,dy,dz);
    glRotated(otoceni,0,0,1);
    glRotated(90.0,1,0,0);
    glutSolidCone(0.1,0.2,20,20);
  glPopMatrix(); 

  /* Krychle */
  glPushMatrix();
    glTranslated(0,0,0.1);
    glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
    glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
    glMaterialfv(GL_FRONT,GL_SHININESS,high_shininess);
    glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);

    glColor3d(0.75,0,0);

    glutSolidCube(0.18);
  glPopMatrix();

  /* Objekt letajici po spirale */
  glPushMatrix();
    if (i<0.5)
	{
      spx = cos (2*PI*pocet_otacek*i*2) * polomer_spiraly;
	  spy = sin (2*PI*pocet_otacek*i*2) * polomer_spiraly;
	  spz = z_min + (i*2)*(z_max - z_min);
	  i += 0.001;
	}
    else
	{
      j = 1 - i;
	  i += 0.001;
	  spx = cos (2*PI*pocet_otacek*j*2) * polomer_spiraly;
	  spy = sin (2*PI*pocet_otacek*j*2) * polomer_spiraly;
	  spz = z_min + (j*2)*(z_max - z_min);
	  if (j <= 0.0) i = 0;
	}
    glTranslated(spx,spy,spz);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, hodn1);
    glMaterialfv(GL_FRONT, GL_SPECULAR, hodn3);
    glColor3d(0.0,0.0,0.75);

    glutSolidSphere(0.08,20,20);
  glPopMatrix();

  /* Podlozka, nad kterou jsou telesa... */
  glPushMatrix();
    glMaterialfv(GL_FRONT, GL_DIFFUSE, hodn1);
    glMaterialfv(GL_FRONT, GL_SPECULAR, hodn2);
    glColor3d(0.0,0.0,0.001);

    glTranslated(0, 0, -0.15);
    glScaled(1.7, 1.7, 0.01);
    glutSolidCube(1);
  glPopMatrix(); 
}

/* Vlastni vytvoreni sceny */
void createScene(void)
{
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslated(0,0,0);
  vytvor();
}

void main_display(void)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-0.1, 0.1, -0.1, 0.1, 0.1, 5);
  /* Osetreni pohybu kamery */
  if (kamera) {
    kx = cos(2*PI*ki*2) * polomer_kamery;
    ky = sin(2*PI*ki*2) * polomer_kamery;
    ki += 0.001;
	if (ki > 1) ki = 0;
    gluLookAt(kx,ky,1, 0,0,0, 0,0,1);
  } else {
	  if (bocni) {
        gluLookAt(0.0,1.0,0.0, 0,0,0, 0,0,1.0);
      } else {
		gluLookAt(0,0,1.5, 0,0,0, 0,1,0);
      };
  }
  createScene();
  glutSwapBuffers();
}

void main_menu(int value) {
    switch (value) {
      case 1: animace=GL_TRUE;
              break;
      case 2: animace=GL_FALSE;
              break;
      case 3: kamera=GL_TRUE;
              break;
      case 4: kamera=GL_FALSE;
              bocni =GL_FALSE;
              break;
      case 5: CloseTheWindow();
              exit(0);
              break;
      case 6: kamera=GL_FALSE;
		      bocni=GL_TRUE;
              break;
    }
}

void main_reshape(int width,  int height) {
   glViewport(0, 0, width, height);
}

void CloseTheWindow() {
  glutDestroyMenu(menuID);
  glutDestroyWindow(windowID);
}


void world_idle(void) {
  glutPostRedisplay();
}

int main(int argc, char** argv) {
  GLfloat pos_lig[] = { 1.5 ,1.5 ,1.5 ,0.0 };
  GLfloat pos_lig2[] = { 0 ,0 ,1.5 ,0.0 };

  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
  glutInitWindowSize(width, height);
  glutInitWindowPosition(100, 100);

  windowID = glutCreateWindow("Zapoctovy priklad do P009 <Marie Cackova>");
  glutReshapeFunc(main_reshape);
  glutDisplayFunc(main_display);

  menuID=glutCreateMenu(main_menu);
    glutAddMenuEntry("Zapnuti pohybu objektu",1);
    glutAddMenuEntry("Vypnuti pohybu objektu",2);
    glutAddMenuEntry("Rotovani pohledu kamerou",3);
    glutAddMenuEntry("Pohled z ptaci perspektivy",4);
    glutAddMenuEntry("Bocni pohled",6);
    glutAddMenuEntry("Konec",5);
    glutAttachMenu(GLUT_RIGHT_BUTTON|GLUT_LEFT_BUTTON);

  glutIdleFunc(world_idle);
  glLightfv(GL_LIGHT0, GL_POSITION, pos_lig);
  glLightfv(GL_LIGHT1, GL_POSITION, pos_lig2);
  glDepthFunc(GL_LEQUAL);
  glColorMaterial(GL_FRONT, GL_DIFFUSE); 
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_DEPTH_TEST);

  glutMainLoop();

  return 0;
}
